@@ -1,17 +1,17 @@
 #' @title Create interactive bars
 #'
 #' @description
-#' The geometries are based on \code{\link[ggplot2]{geom_bar}}
-#' and \code{\link[ggplot2]{geom_col}}.
+#' The geometries are based on [geom_bar()]
+#' and [geom_col()].
 #' See the documentation for those functions for more details.
 #'
 #' @param ... arguments passed to base function,
-#' plus any of the \code{\link{interactive_parameters}}.
+#' plus any of the [interactive_parameters()].
 #' @inheritSection interactive_parameters Details for geom_*_interactive functions
 #' @examples
 #' # add interactive bar -------
 #' @example examples/geom_bar_interactive.R
-#' @seealso \code{\link{girafe}}
+#' @seealso [girafe()]
 #' @export
 geom_bar_interactive <- function(...)
   layer_interactive(geom_bar, ...)
@@ -29,7 +29,7 @@ GeomInteractiveBar <- ggproto(
     gr <- GeomBar$draw_key(data, params, size)
     add_interactive_attrs(gr, data, data_attr = "key-id")
   },
-  draw_panel = function(self, data, panel_params, coord, width = NULL) {
+  draw_panel = function(self, data, panel_params, coord, width = NULL, flipped_aes = FALSE) {
     GeomInteractiveRect$draw_panel(data, panel_params, coord)
   }
 )
