@@ -7,6 +7,8 @@ public:
   std::string svg_fill_attr();
   std::string svg_stroke_attr();
   int is_transparent();
+  std::string color();
+  std::string opacity();
 
 private:
   int col;
